import os
from rocolib.library import getComponent

from known_models import known_models


def make_thumbnails(models, thumbs_dir, force_update=False):
    toCreate = set(models.keys())
    if not force_update:
        extant_models = [file.split('.')[0] for file in os.listdir(thumbs_dir)]
        print('Thumnails exist for: ', extant_models)
        toCreate -= set(extant_models)

    if len(toCreate) > 0:
        print('Models to be created: ', toCreate)
    else:
        print('No models will be created!')
        print('  Tip: Set `force_update=True` in make_thumbnails(), to force refresh thumbnails')

    for component in toCreate:
        print(f"Thumbnailing {component}...")
        params = models.get(component, {})
        try:
            f = getComponent(component)
            thickness = params.get("t", 0)
            f.makeOutput(f"{thumbs_dir}/{component}", 
                default=False, png=True, 
                thickness=thickness, joint=params.get("joint", None))
        except Exception as e:
            print(f"Error trying to make {component}:")
            print(repr(e))
    print('\n>>> Done creating thumbnails for: ', toCreate)

if __name__ == '__main__':
    # Thumb directory is relative to current file
    static_thumbs_dir = \
      os.path.dirname(os.path.realpath(__file__)) + \
      "/static/img/model_thumbnails"

    for path in [f'{static_thumbs_dir}', '/tmp']:
        if not os.path.exists(path):
            print(f'>> Making directory: {path}')
            os.makedirs(path)

    models = known_models
    force_update = False

    ### For debugging only:
    # models = {"Stool": {"t": 3, "joint": None}}
    force_update=True

    make_thumbnails(models, static_thumbs_dir, force_update)
    print(f'\n>>> Generated Thumbnails in path: {static_thumbs_dir}')
