from rocolib.api.component import Component
# We're making a component
c = Component()

# ... 6 repeated squares

c.addSubcomponent("face1","Rectangle")
c.addSubcomponent("face2","Rectangle")
c.addSubcomponent("face3","Rectangle")
c.addSubcomponent("face4","Rectangle")
c.addSubcomponent("face5","Rectangle")
c.addSubcomponent("face6","Rectangle")

# I pick one parameter, which is the lenght square side
# let's say default = 50mm

c.addParameter("length", 50)
c.addParameter("tabWidth")

# We need to constrain all parameters of all subcomponents to be functions of the parameter of our new component
# Simple syntax because the function here is the identity function

c.addConstraint(("face1", "l"), "length")
c.addConstraint(("face1", "w"), "length")

c.addConstraint(("face2", "l"), "length")
c.addConstraint(("face2", "w"), "length")

c.addConstraint(("face3", "l"), "length")
c.addConstraint(("face3", "w"), "length")

c.addConstraint(("face4", "l"), "length")
c.addConstraint(("face4", "w"), "length")

c.addConstraint(("face5", "l"), "length")
c.addConstraint(("face5", "w"), "length")

c.addConstraint(("face6", "l"), "length")
c.addConstraint(("face6", "w"), "length")

# pick out matching edges
# (b2-c4) should fold at 90degrees (mountain fold, same for all folds)
# I think positive are valley folds, negatives are mountain folds

c.addConnection(("face1", "t"), ("face2", "b"), angle=-90)
c.addConnection(("face1", "l"), ("face3", "r"), angle=-90)
c.addConnection(("face1", "r"), ("face4", "l"), angle=-90)
c.addConnection(("face1", "b"), ("face5", "t"), angle=-90)
c.addConnection(("face5", "b"), ("face6", "t"), angle=-90)

# in the folded state, (xx-yy) edges should be connected
# We'll hold the edges together with tab/slot connections
# The first argument will have a 10mm tab, the second will have the slot

c.addConnection(("face5", "l"), ("face3", "b"), angle=-90, tabWidth=10)
c.addConnection(("face3", "t"), ("face2", "l"), angle=-90, tabWidth=10)
c.addConnection(("face2", "r"), ("face4", "t"), angle=-90, tabWidth=10)
c.addConnection(("face4", "b"), ("face5", "r"), angle=-90, tabWidth=10)

c.addConnection(("face6", "l"), ("face3", "l"), angle=-90, tabWidth=10)
c.addConnection(("face6", "b"), ("face2", "t"), angle=-90, tabWidth=10)
c.addConnection(("face6", "r"), ("face4", "r"), angle=-90, tabWidth=10)

# And we're done!  We can make it right here right now, and the design files will end up in ./output/die/*

c.makeOutput("output/die50mm", tree=True, display=False)

# Or we can save it to our library to be used later

# XXX: check the path to make sure it saves to the right folder rocolib/library/
# TODO: The absolute path to the library folder should be a constant in rocolib.library
#       toYaml should have two arguments: 
#           the name of the yaml file e.g. "Die", and 
#           the folder to save it to, defaults to the constant in rocolib.library

c.toYaml("library/Die.yaml")

###
### ... some time later, in a different python file
###

# Let's make a larger die from our saved component 

from rocolib.library import getComponent
d = getComponent("Die", length=65)
d.makeOutput("output/die65mm", tree=True, display=False)
