if (!Detector.webgl) {
    Detector.addGetWebGLMessage();
}

var stl_container = document.getElementById('stl');

// All of these variables will be needed later, just ignore them for now.
var camera, controls, scene, renderer;
var lighting, ambient, keyLight, fillLight, backLight;
var stl_width = stl_container.offsetWidth; //h & w of div element, incl. pad & border
var stl_height = stl_container.offsetHeight;
var stl_height = stl_height* 1.1;
// var width = window.innerWidth;
//var stl_width = container.style.width;
//var stl_width = container.clientWidth;
stlinit();
render();

function stlinit() {
    // RENDERER
    renderer = new THREE.WebGLRenderer( { alpha: true } );
    renderer.setPixelRatio(window.devicePixelRatio);
    renderer.setSize(stl_width, stl_height);
    renderer.gammaInput = true;
    renderer.gammaOutput = true;
    renderer.shadowMap.enabled = true;

    // CAMERA
    camera = new THREE.PerspectiveCamera(45, stl_width / stl_height, 1, 100000);
    camera.position.z = 500;

    // SCENE
    scene = new THREE.Scene();
    // scene.background = new THREE.Color( 0x72645b );

    // LIGHTING
    scene.add( new THREE.HemisphereLight( 0x443333, 0x111122 ) );
    addShadowedLight( 1000, 1000, 1000, 0xffffff, 1.35 );
    addShadowedLight( 500, 1000, -1000, 0xffaa00, 1 );

    // CONTROLS
    controls = new THREE.OrbitControls(camera, renderer.domElement);
    controls.enableDamping = true;
    controls.dampingFactor = 0.25;
    controls.enableZoom = true;
    controls.enableKeys = false;

    stl_container.appendChild(renderer.domElement);
}

function addShadowedLight( x, y, z, color, intensity ) {
    var directionalLight = new THREE.DirectionalLight( color, intensity );
    directionalLight.position.set( x, y, z );
    scene.add( directionalLight );
    directionalLight.castShadow = true;
    var d = 1;
    directionalLight.shadow.camera.left = -d;
    directionalLight.shadow.camera.right = d;
    directionalLight.shadow.camera.top = d;
    directionalLight.shadow.camera.bottom = -d;
    directionalLight.shadow.camera.near = 1;
    directionalLight.shadow.camera.far = 4;
    directionalLight.shadow.mapSize.width = 1024;
    directionalLight.shadow.mapSize.height = 1024;
    directionalLight.shadow.bias = -0.002;
}

var mesh;
var material = new THREE.MeshPhongMaterial( { color: 0xff5533, specular: 0x111111, shininess: 200 } );
var loader = new THREE.STLLoader();

function stlload(stlmodel) {
    // OBJECT

    geometry = loader.parse( stlmodel );

    mesh = new THREE.Mesh( geometry, material );
    mesh.castShadow = true;
    mesh.receiveShadow = true;
    mesh.scale.set(1000, 1000, 1000);

    geometry.computeFaceNormals();
    geometry.computeVertexNormals();

    scene.add( mesh );
}

function stlunload() {
    scene.remove(mesh);
}

function stlreload(stlmodel) {
    stlunload();
    stlload(stlmodel);
    render();
}

function render() {
    requestAnimationFrame(render);
    controls.update();
    renderer.render(scene, camera);
}

var paramform = document.getElementById('paramform');
paramform.addEventListener("submit", function (event) {
    event.preventDefault();
    remake();
});

function appendParamHistory() {
    formvalues = [];
    $('#params input[id]').map(function() {
        if (this.value) {
            fielditem = this.id[0] + ': ' + this.value  ;
            //fielditem = this.id + ': ' + this.value  ;
            formvalues.push(fielditem);
        }
        else { formvalues.push(this.id + ': n/a');
        };

    });
    hist = document.getElementById("history");
    histstring = '<li>' + String(formvalues.join(', ')) + '</li>'; // force String for some minimal safety
    //hist.innerHTML += histstring;
    console.log('attempted to append history');
    hist.insertAdjacentHTML('afterbegin', histstring)

}

paramform.addEventListener("onchange", function (event) {
    remake();
});

function svgreload(svgmodel) {
  swapSVG(document.getElementById('dxf'), svgmodel);
}

function directlink(ext) {
  var formdata = new FormData(paramform);
  url = component + "." + ext + "?" + (new URLSearchParams(formdata)).toString();
  return url;
}

function makelink(par, ext, data) {
      filename = component + "." + ext;

      var element = document.createElement('a');
    //   element.className = 'card-link col-4 '
      element.className = 'btn btn-outline-primary btn-md mr-1 '
      element.innerHTML = ext;
    //   element.innerHTML = "Download ." + ext;
      //element.setAttribute('download', filename);
      element.href = directlink(ext);
      // element.setAttribute('href', 'data:text/plain;charset=utf-8,' + encodeURIComponent(data));

      par.appendChild(element);
      //par.appendChild(document.createElement('br'));
}

function makelinks(data) {

    var par = document.getElementById('downloads');
    par.innerHTML = "";

    makelink(par, "stl", data["stl"]);
    makelink(par, "dxf", data["silhouette"]);
    makelink(par, "svg", data["unfolding"]);
}

var stlarray = [];

function animvar(inpt) {
    function remake_callback(data) {
        stlarray.push(data["stl"])
        i = parseFloat(inpt.value) + parseFloat(inpt.step)
        if (i < parseFloat(inpt.max)) {
            inpt.value = i;
            remake(remake_callback);
        }
    }

    function anim_callback(item, index) {
        setTimeout(() => {stlreload(item);}, 10 * index);
    }
        
    if (stlarray.length) {
        stlarray.forEach(anim_callback)
    } else {
        remake(remake_callback);
    };
}
        
function remake(callback) {
  var xhr = new XMLHttpRequest();
  var formdata = new FormData(paramform);

  // Define what happens on successful data submission
  xhr.addEventListener("load", function(event) {
    data = JSON.parse(event.target.responseText);
    makelinks(data);
    stlreload(data["stl"]);
    svgreload(data["unfolding"]);
    if (callback) callback(data);
    appendParamHistory();
  });

  // Define what happens in case of error
  xhr.addEventListener("error", function(event) {
    console.log('Oops! Something went wrong.');
  });

  xhr.open('POST', window.location.pathname + ".ajax");
  xhr.send(formdata);
}

window.onload = function(e){
    remake();
}



var dxf_container = document.getElementById('dxf');
//var dxf_height = stl_height; 

function dxfinit() {
    if (typeof dxfmodel == "undefined") 
       fdxfmodel = 'static/graph-silhouette.dxf';

    var dxfscene = new THREE.Scene();
    dxfrenderer = new THREE.WebGLRenderer( );
    //dxfrenderer.setSize(dxf_width, dxf_height);
    dxf_container.appendChild(dxfrenderer.domElement);

    var loader = new THREE.DXFLoader()
    loader.load( dxfmodel, function ( data ) {
        dxfscene.add(data);
    })
}

// dxfinit();
